@@ -2,6 +2,9 @@
 #ifndef MPG123_ICY2UTF_H
 #define MPG123_ICY2UTF_H
 
-char *icy2utf8(const char *);
+#ifndef NO_ICY
+/* (string, force conversion) */
+char *icy2utf8(const char *, int);
+#endif
 
 #endif
